use actix_identity::Identity;
// TODO: Guard for login
use actix_web::{get, post, put, web, HttpResponse, Responder};
use serde::Deserialize;

use crate::{model::user::User, repo::user_repository::UserRepository};

#[derive(Debug, Deserialize)]
pub struct NewUser {
    pub login: String,
    pub password: String,
}

// TODO: Only allow if role is admin (If I implement roles for this evaluation...)
/// Returns a json list of all users.
#[get("/user")]
pub async fn get_user(_id: Identity, repo: web::Data<UserRepository>) -> impl Responder {
    match repo.read_all() {
        Ok(users) => HttpResponse::Ok().json(users),
        Err(err) => {
            log::error!("Could not read from user repo: {err}");
            HttpResponse::InternalServerError()
                .body(format!("Could not read from user repo: {err}"))
        }
    }
}

// REMARK: This uses the non-JSON way of inserting into cassandra
// this is why we need two separate REST endpoints to do the creates/updates
// whereas the todo api only requires PUT (it uses the JSON INSERT in cassandra)
// TODO: Only admin users should be able to modify other users but users should be
// able to modify themself -> Requires roles to be implemented
/// Updates an existing user.
#[put("/user")]
pub async fn put_user(
    _id: Identity,
    payload: web::Json<User>,
    repo: web::Data<UserRepository>,
) -> impl Responder {
    log::debug!("Received {payload:?}");
    match repo.update(&payload.0) {
        Ok(_) => HttpResponse::Ok().finish(),
        Err(err) => {
            let msg = format!("{err}: {payload:?}");
            log::debug!("{}", msg);
            HttpResponse::InternalServerError().body(msg)
        }
    }
}

// TODO: For a real app, impl smth like a registration-secret or email verification
/// Creates a new user.
#[post("/user")]
pub async fn post_user(
    payload: web::Json<NewUser>,
    repo: web::Data<UserRepository>,
) -> impl Responder {
    log::debug!("Received {payload:?}");
    let user = User::new(
        &payload.login,
        &payload.password,
        &sha256::digest(format!("{}{}", payload.login, &payload.password)),
    );
    match repo.create(&user) {
        Ok(_) => {
            log::debug!("Successfully created {user:?}");
            HttpResponse::Created().finish()
        }
        Err(err) => {
            log::debug!("{err}");
            HttpResponse::BadRequest().body(format!("{user:?}"))
        }
    }
}
