import { useNavigate } from '@solidjs/router';
import { Component, createEffect, createSignal, Show } from 'solid-js';
import RestClient from '../api/RestClient';
import { setUser, User } from '../App';
import Button from '../ui/Button';

type UserRequest = {
  login: string;
  password: string;
};

const Login: Component = () => {
  const nav = useNavigate();
  const [userRequest, setUserRequest] = createSignal<UserRequest>({
    login: '',
    password: '',
  });
  const [login, setLogin] = createSignal('');
  const [password, setPassword] = createSignal('');

  // Populate the current user outside the JSX (we need createEffect for this!)
  createEffect(async () => {
    if (userRequest().login.trim() === '' || userRequest().password.trim() === '') {
      return;
    }
    if (window.location.href.includes('/register')) {
      const register = await RestClient.POST('/user', JSON.stringify(userRequest()));
      if (register) {
        console.log('Could not create new user');
        return;
      }
    }
    const user = (await RestClient.POST('/login', JSON.stringify(userRequest()))) as User;
    if (user.id === undefined) {
      console.log(user);
      return;
    }
    setUser(user);
    nav('/');
  });

  return (
    <div class='grid h-screen place-items-center'>
      <div class='block p-6 rounded-lg shadow-lg bg-white max-w-sm'>
        <form>
          <div class='form-group mb-6'>
            <label for='username' class='form-label inline-block mb-2 text-gray-700'>
              Login
            </label>
            <input
              type='text'
              class='form-control
        block
        w-full
        px-3
        py-1.5
        text-base
        font-normal
        text-gray-700
        bg-white bg-clip-padding
        border border-solid border-gray-300
        rounded
        transition
        ease-in-out
        m-0
        focus:text-gray-700 focus:bg-white focus:border-blue-600 focus:outline-none'
              id='username'
              placeholder='Username'
              onInput={(e) => setLogin(e.currentTarget.value)}
              value={login()}
            />
          </div>
          <div class='form-group mb-6'>
            <label for='Password' class='form-label inline-block mb-2 text-gray-700'>
              Password
            </label>
            <input
              type='password'
              class='form-control block
        w-full
        px-3
        py-1.5
        text-base
        font-normal
        text-gray-700
        bg-white bg-clip-padding
        border border-solid border-gray-300
        rounded
        transition
        ease-in-out
        m-0
        focus:text-gray-700 focus:bg-white focus:border-blue-600 focus:outline-none'
              id='Password'
              placeholder='Password'
              onInput={(e) => setPassword(e.currentTarget.value)}
              value={password()}
            />
          </div>
          <Show
            when={window.location.href.includes('/register')}
            fallback={
              <>
                <Button
                  fullWidth
                  onClick={() => {
                    console.log(login(), password());
                    setUserRequest({ login: login(), password: password() });
                  }}
                >
                  Sign in
                </Button>
                <p class='text-gray-800 mt-6 text-center'>
                  Want to track your todo{"'"}s?{' '}
                  <a
                    href='/register'
                    class='text-blue-600 hover:text-blue-700 focus:text-blue-700 transition duration-200 ease-in-out'
                  >
                    Register
                  </a>
                </p>
              </>
            }
          >
            <Button
              fullWidth
              onClick={() => {
                console.log(login(), password());
                setUserRequest({ login: login(), password: password() });
              }}
            >
              Register
            </Button>
            <p class='text-gray-800 mt-6 text-center'>
              Back to login?{' '}
              <a
                href='/login'
                class='text-blue-600 hover:text-blue-700 focus:text-blue-700 transition duration-200 ease-in-out'
              >
                Sign in
              </a>
            </p>
          </Show>
        </form>
      </div>
    </div>
  );
};

export default Login;
