import { Component, JSX } from 'solid-js';

type ButtonProps = {
  backgroundColor?: string;
  color?: string;
  fullWidth?: boolean;
  onClick?: Function;
  children?: JSX.Element;
};

const backgroundColors: { [key: string]: string } = {
  blue: 'bg-sh-blue hover:bg-sh-blueM1 focus:bg-sh-blueM1 active:bg-sh-blueM1',
  red: 'bg-red-600 hover:bg-red-700 focus:bg-red-700 active:bg-red-800',
  green: 'bg-green-600 hover:bg-green-700 focus:bg-green-700 active:bg-green-800',
  yellow: 'bg-sh-yellow hover:bg-sh-yellowM1 focus:bg-sh-yellowM1 active:bg-sh-yellowM1',
  magenta:
    'bg-sh-magenta hover:bg-sh-magentaM1 focus:bg-sh-magentaM1 active:bg-sh-magentaM1',
  gray: 'bg-gray-700 hover:bg-gray-800 focus:bg-gray-800 active:bg-gray-800',
  cyan: 'bg-cyan-700 hover:bg-cyan-900 focus:bg-cyan-900 active:bg-cyan-800',
};

const colors: { [key: string]: string } = {
  white: 'text-white',
  black: 'text-black',
};

const Button: Component<ButtonProps> = (props) => {
  const backgroundColor = backgroundColors[props.backgroundColor || 'blue'];
  const color = colors[props.color || 'white'];
  const fullWidth = props.fullWidth ? 'w-full' : '';
  const fun =
    props.onClick !== undefined
      ? (e: MouseEvent) => {
          e.preventDefault();
          (props.onClick as () => void)();
        }
      : (e: MouseEvent) => {
          e.preventDefault();
          console.log(
            'Default behavior of Button component is invoked!',
            'You might want to change the onClick function?'
          );
        };
  const btnStyle = `${color} ${backgroundColor} ${fullWidth} inline-block px-6 py-2.5 font-medium text-xs leading-tight uppercase rounded-full shadow-md  hover:shadow-lg  focus:shadow-lg focus:outline-none focus:ring-0  active:shadow-lg transition duration-150 ease-in-out`;

  return (
    <button class={btnStyle} onClick={fun}>
      {props.children}
    </button>
  );
};

export default Button;
