import { Component, For, JSX } from 'solid-js';

type TableProps = {
  data: object[];
};

type TableRowProps = {
  children: JSX.Element;
};

type TableDataProps = {
  children?: JSX.Element;
};

type TableHeadProps = {
  children?: JSX.Element;
};

export const TableHead: Component<TableHeadProps> = (props) => {
  return (
    <th scope='col' class='text-sm font-bold px-6 py-4'>
      {props.children}
    </th>
  );
};

export const TableData: Component<TableDataProps> = (props) => {
  return <td class='text-sm text-gray-900 font-light px-6 py-4 '>{props.children}</td>;
};

export const TableRow: Component<TableRowProps> = (props) => {
  const rowClass = 'border-b';
  return <tr class={rowClass}>{props.children}</tr>;
};

// Would have been nicer with cards instead of a table...
const Table: Component<TableProps> = (props) => {
  if (props.data.length < 1) {
    return <></>;
  }
  return (
    <div class='overflow-hidden mt-10 mx-10'>
      <table class='min-w-full text-center'>
        <thead class='border-b text-sh-bgM1 bg-sh-yellow'>
          {Object.keys(props.data[0]).map((key) => (
            <TableHead>{key}</TableHead>
          ))}
        </thead>
        <tbody>
          <For each={props.data}>
            {(child) => (
              <TableRow>
                {Object.values(child).map((val) => (
                  <TableData>{val}</TableData>
                ))}
              </TableRow>
            )}
          </For>
        </tbody>
      </table>
    </div>
  );
};

export default Table;
